﻿# ******************************************************************************************
#	KnifeP3N Python
#		by Schubaal
#		5.0.1.2 testable
#
# * Description:
#	- when there's only one player left on each team, knife battles ensue!
#
# * Install instructions:
#			1. Install Mattie's EventScripts 2+:
#				http://mattie.info/cs
#
#			2. Extract the file [knifep3n.zip] to your /cstrike/ directory:
#				/addons/eventscripts/knifep3n/knifep3n.py
#				/sound/ql/rocky.mp3
#				/sound/ql/chicken.wav
#				/sound/ql/DaffyDoit.wav
#				/sound/ql/blip1.wav
#
#			3. Add the following line somewhere in autoexec.cfg or server.cfg:
#				es_load knifep3n
#
#			4. Adjust any settings in the class KnifeP3N_Settings below.
#
# ******************************************************************************************

import es
import popuplib
import playerlib
import random
import services
import gamethread
import weaponlib
import string

knifep3nVersion = "5.0.1.2"
versionDate = "2010-02-10 09:26 EST"
es.set('knifep3n', knifep3nVersion)
es.makepublic('knifep3n')

info = es.AddonInfo()
info.name		 = "KnifeP3N Python"
info.version	 = knifep3nVersion
info.url		 = "http://forums.mattie.info/cs/forums/viewtopic.php?t=19090"
info.basename	 = "knifep3n"
info.author		 = "Schubaal"
info.description = "Based on the ESClassic script by P3N."

class knifep3n_Settings:
	def __init__(self):
		# *************************************************
		''' Message Filter '''
		# 0 = send all messages
		# 1 = send only important messages
		# 2 = send no messages
		# Default: 0
		self.chat_filter = 0
		# not yet implemented
		# *************************************************
		''' Fight Health '''
		# Sets how much health players get at the beginning of fights
		# Default: 100
		self.fight_health = 100
		# *************************************************
		''' Beacon '''
		# Enables a ring beacon on each player when fight starts
		# Default = 1
		self.beacon = 1
		# *************************************************
		''' Beacon sound '''
		# Sets what sound emits from players with the beacon
		# Default = "ql/blip1.wav"
		self.beacon_sound = "ql/blip1.wav"
		# *************************************************
		''' Beacon color '''
		# Uses HEX values to set beacon color (0 min, 255 max)
		# green + blue = cyan (0 255 255)
		# red + blue = magenta (255 0 255)
		# red + green = yellow (255 255 0)
		# TERRORIST (default = "255 0 0")
		self.t_beacon = "255 0 0"
		# COUNTER-TERRORIST (default = "0 0 255")
		self.ct_beacon = "0 0 255"
		# *************************************************
		''' Beacon rate '''
		# Sets how often the beacon occurs (in seconds)
		self.beacon_rate = 2.0
		# *************************************************
		''' Beacon radius '''
		self.beacon_size = 800
		# *************************************************
		''' Bot voting '''
		# Sets how bots will vote
		# ALWAYS    NO = 0
		# ALWAYS   YES = 1
		# 50/50 YES/NO = 2
		# 33/66 YES/NO = 3
		# 25/75 YES/NO = 4
		self.bot_vote = 1
		# *************************************************
		''' Sounds '''
		# Put the exact file name here. Make sure there are no 
		# spaces in it. Rename the file if you have to remove 
		# any spaces in the name. Put your sounds in the 
		# "/cstrike/sound/" directory. Remember to include the name
		# of a subdirectory (such as "ql") in the variable if the
		# sound is within a subdirectory.
		#
		# To disable a particular sound, leave it empty between the 
		# quotes, ""
		#
		# Default sound files:
		# Votes yes ""
		# Voted no "ql/chicken.wav"
		# Start fight "ql/rocky1.mp3"
		# Winner of fight "ql/DaffyDoit.wav"
		#
		# sounds_start and sounds_winner can support
		# multiple sound files, and one will be played at random
		# ex: "ql/rocky1.mp3,ql/example1.mp3,ql/example2.mp3"
		#
		# Remember that each additional file will add to the time it
		# takes for users to download from your server.
		self.sounds_yes = ""
		self.sounds_no = "ql/chicken.wav"
		self.sounds_start = "ql/rumble.mp3,ql/rocky1.mp3"
		self.sounds_winner = "ql/DaffyDoit.wav"
		# *************************************************
		''' Timer '''
		# sets how many seconds fights last until they're automatically ended
		self.timer = 30
		# *************************************************
		''' Awards '''
		# Give bonuses/detriments to the winner of fights
		# 1 = on, 0 = off
		self.award_money = 0
		self.award_health = 1
		self.award_speed = 1
		# How much money/health/speed? 
		# Money is how much cash to add
		# Health is how much health they'll spawn with (NOT how much is added)
		# Speed is a multiplier, so 2.0 is 2 times as fast
		self.bonus_money = 16000
		self.bonus_health = 120
		self.bonus_speed = 1.1
		''' resilience '''
		# If enabled, the winner will keep his award abilities until the next fight
		self.resilience = 0
		# *************************************************
		''' Teleport '''
		# Teleports players to be closer together at the start of a fight
		self.teleport = 1
		''' Slap '''
		# Helps prevent players from being teleported into each other
		# this is only used if teleport is enabled
		self.slap = 0
		# slap not yet implemented
		# *************************************************
		''' Spoils of War '''
		# If enabled, the winner will get his weapons back the following round
		# Don't use this if you're running an addon like GunGame
		self.keep_weapons = 0
		# slap not yet implemented
		# *************************************************
		''' Force Fight '''
		# If enabled, players have no choice but to knife fight, otherwise they can choose
		self.no_vote = 0
		# slap not yet implemented
		# *************************************************
		''' Punish Chickens '''
		# If enabled, punishes players who decline a knife fight
		self.punishment = 0
		''' Chicken Color '''
		# Sets the color of a chicken (255 255 255 is normal color)
		# green + blue = cyan (0 255 255)
		# red + blue = magenta (255 0 255)
		# red + green = yellow (255 255 0)
		#                     R     G    B
		# Default = 255 255  0  (yellow)
		self.chicken_color = "255 255 0"
		''' Chicken Speed '''
		# Multiplies the speed of a chicken (i.e. - 2.0 is 2 times normal speed)
		self.chicken_speed = 0.8
		# slap not yet implemented
		# *************************************************
		''' Punish Suicide '''
		# when a player kills him or herself, that's lame
		# also used when time runs out and either one or both players die
		# Sets how much health these players spawn with the following round
		self.lame_health = 50
		# *************************************************
		''' Predator Skin '''
		# Temporarily removed. I don't feel it's optimal to implement the old predator skin in v5
		#	until I have the time to work out allowing custom skins.
		# *************************************************

#**************************************************************
#   Do not edit below this line unless you really know what you're doing!
#**************************************************************


class EsToolsWrapper:
	def beacon(self, userid):
		x, y, z = playerlib.getPlayer(userid).get('location')
		z = z+10
		team = playerlib.getPlayer(userid).team
		if team == 3:
			es.server.cmd('es est_effect 10 #a 0 "sprites/lgtning.vmt" '+str(x)+' '+str(y)+' '+str(z)+' 1 '+str(settings.beacon_size)+' 1 10 50 0 '+str(settings.t_beacon)+' 200 0')
		elif team == 2:
			es.server.cmd('es est_effect 10 #a 0 "sprites/lgtning.vmt" '+str(x)+' '+str(y)+' '+str(z)+' 1 '+str(settings.beacon_size)+' 1 10 50 0 '+str(settings.ct_beacon)+' 200 0')
		else:
			es.msg('#multi',knifep3n_msg+"This message shouldn't ever be sent.")

if es.exists('variable', 'est_version'):
	est = EsToolsWrapper()
else:
	est = 0


class knifep3n_Player:
	def __init__(self,userid):
		self.userid = int(userid)
		self.Player = playerlib.getPlayer(userid)
		'''
		if settings.keep_weapons:
			self.weaponPrimary = 0
			self.weaponSecondary = 0
			self.weaponHE = 0
			self.weaponFlash = 0
			self.weaponSmoke = 0
			self.weaponDefuse = 0
			self.weaponKevlar = 0
		'''

######################
# Status works in phases:
status = 0
# 0 = not started
# 1 = popup sent
# 2 = player 1 voted yes
# 3 = both players voted
# 4 = fight in progress
# 5 = player voted no
######################
settings = knifep3n_Settings()
player1 = 0
player2 = 0
winner = 0

def player_vote(userid,pchoice,pname):
	if pchoice == 1:
		vote_yes(userid)
	if pchoice == 2:
		vote_no(userid)

# Vote menu
menu = popuplib.create('knifep3n_menu')
menu.addline('www.clan-gangsta.ucoz.ru Knife fight?', 'en')
menu.addlineAll('------------------------------')
menu.addline('->1. Yes, of course!', 'en')
menu.addline("->2. No, I'm a chicken...", 'en')
menu.addlineAll('------------------------------')
menu.menuselect = player_vote

knifep3n_msg = "[#greenKnifeP3N#default]#lightgreen "

str_voting = "Knife vote in progress."
str_accepted = " has accepted the knife fight!"
str_chicken = " is a chicken!"
str_agree = "Both players have accepted the knife fight!"
str_forced = "Knife fight!"
str_teleport = " will be teleported in %s seconds!"
str_teleport1 = " will be teleported in 1 second!"
str_suicide = " committed suicide, that's lame!"
str_cheated_early = " cheated! Killed opponent before the fight started!"
str_cheated_knife = " cheated! Did not use a knife!"
str_money = "Winner will receive %s in cash!"%str(settings.bonus_money)
str_health = "Winner will spawn with %s health!"%str(settings.bonus_health)
str_speed = "Winner will spawn with speed multiplied by %s"%str(settings.bonus_speed)
str_inprogress = " has been slain for spawning during a knife fight."
str_whyfight = "Why don't you fight, chickens?"
str_won = " won the knife fight! He's the man!"

def load():
	'''
	Loads KnifeP3N assist scripts, sets popup menu, runs init
	'''
	init()
	es.server.cmd('clientcmd create say @kp_force knifep3n/kp_helper/force_fight "knifep3n" #admin')
	es.msg('#multi', knifep3n_msg+" version "+knifep3nVersion+" has been loaded.")

def force_fight():
	global started, player1, player2
	if not settings.no_vote:
		if es.getlivingplayercount(2) == 1 and es.getlivingplayercount(3) == 1:
			for player in playerlib.getUseridList("#alive"):
				if not player1:
					player1 = knifep3n_Player(player)
				else:
					player2 = knifep3n_Player(player)
					es.msg('#multi',knifep3n_msg+str_forced)
					status = 3
					prep_fight()
		else:
			es.tell(es.getcmduserid(), '#multi', '#green[KnifeP3N]#lightgreen There must be one player alive on each team to force a fight!')
	else:
		es.tell(es.getcmduserid(), '#multi', "#green[KnifeP3N]#lightgreen No vote is already on, there's no need to force a fight manually.")

def es_map_start(event_var):
	'''
	Simply runs init
	'''
	init()

def init():
	'''
	Sets downloadables
	'''
	# Load sounds into downloadables
	if settings.beacon_sound != "":
		es.stringtable('downloadables', 'sound/'+settings.beacon_sound)
	if settings.sounds_yes != "":
		es.stringtable('downloadables', 'sound/'+settings.sounds_yes)
	if settings.sounds_no != "":
		es.stringtable('downloadables', 'sound/'+settings.sounds_no)
	if settings.sounds_start != "":
		if (',' in settings.sounds_start):
			parts = settings.sounds_start.split(',')
			for k in parts:
				es.server.cmd('es_xmath knifep3n_start_count + 1')
				es.stringtable('downloadables', 'sound/'+str(k))
		else:
			es.stringtable('downloadables', 'sound/'+settings.sounds_start)
	if settings.sounds_winner != "":
		if (',' in settings.sounds_winner):
			for k in parts:
				es.server.cmd('es_xmath knifep3n_winner_count + 1')
				es.stringtable('downloadables', 'sound/'+str(k))
		else:
			es.stringtable('downloadables', 'sound/'+settings.sounds_winner)

def unload():
	'''
	clears the popup
	'''
	menu.unsend(playerlib.getUseridList('#all'))
	popuplib.delete('knifep3n_menu')
	es.unregsaycmd('@kp_force')

def round_start(event_var):
	global winner
	if winner:
		if settings.award_money:
			x = winner.getCash() + winner.settings.bonus_money
			if x > 16000:
				x = 16000
			winner.setCash(x)
		if settings.award_health:
			winner.setHealth(settings.bonus_health)
		if settings.award_speed:
			winner.setSpeed(settings.bonus_speed)
		if not settings.resilience:
			winner = 0
	kp_reset()

def kp_reset():
	global status, player1, player2
	status, player1, player2 = 0,0,0
	menu.unsend(playerlib.getUseridList('#all'))

def bot_vote(botid):
	if settings.bot_vote:
		i = random.randint(1,settings.bot_vote)
		if settings.bot_vote == i:
			vote_yes(botid)
		else:
			vote_no(botid)
	else:
		vote_no(botid)

def vote_yes(userid):
	global player1, player2, status
	if settings.sounds_yes != "":
		if string.find(settings.sounds_yes, ','):
			sounds = string.split(settings.sounds_yes, ',')
			x = len(sounds)
			y = random.randint(0, x-1)
			es.cexec_all("play","1.0",sounds[y])
		else:
			es.cexec_all("play","1.0",settings.sounds_yes)
	if not player1:
		player1 = knifep3n_Player(userid)
		es.msg('#multi', knifep3n_msg+player1.Player.name+str_accepted)
		status = 2
	else:
		player2 = knifep3n_Player(userid)
		status = 3
		prep_fight()

def vote_no(userid):
	es.msg('#multi', knifep3n_msg+str_chicken)
	if settings.sounds_no != "":
		if string.find(settings.sounds_no, ','):
			sounds = string.split(settings.sounds_no, ',')
			x = len(sounds)
			y = random.randint(0, x-1)
			es.cexec_all("play","1.0",sounds[y])
		else:
			es.cexec_all("play","1.0",settings.sounds_no)
	if settings.punishment:
		#chicken_punish(userid)
		pass
	kp_reset()

def force_fight():
	# need to set player1 and player2 here
	prep_fight()

def prep_fight():
	global status
	player1.Player.set('health', settings.fight_health)
	player2.Player.set('health', settings.fight_health)
	for index in weaponlib.getIndexList('#all'):
		es.server.cmd('es_xremove %s' % index)
	for player in playerlib.getUseridList('#alive'):
		gamethread.delayed(0.1, es.server.cmd, 'es_xgive %s item_assaultsuit'%str(player))
		gamethread.delayed(0.1, es.server.cmd, 'es_xgive %s weapon_knife'%str(player))
		es.delayed(0.15, es.sexec(player, 'use weapon_knife'))
	if settings.teleport:
		x,y,z = player1.Player.get('location')
		start_teleport(x,y,z)
	else:
		start_fight_init()

def player_slap():
	pass

def start_teleport(x,y,z):
	if settings.slap:
		player_slap(userid)
	es.centermsg(player2.Player.name+str_teleport%'3')
	gamethread.delayed(1, es.centermsg, (player2.Player.name+str_teleport%'2'))
	gamethread.delayed(2, es.centermsg, (player2.Player.name+str_teleport1))
	gamethread.delayed(3, player_teleport, (x,y,z))
	gamethread.delayed(3, start_fight_init)

def player_teleport(x,y,z):
	es.server.cmd('es_xsetpos %s %s %s %s' % (player2.userid, x, y, z))

def beacon(userid):
	if status == 4:
		est.beacon(userid)
		if settings.beacon_sound != "":
			es.emitsound("player", userid, settings.beacon_sound, 1.0, 0.7)
		gamethread.delayed(settings.beacon_rate, beacon, userid)

def start_fight_init():
	global status
	status = 4
	if settings.sounds_start != "":
		if string.find(settings.sounds_start, ','):
			sounds = string.split(settings.sounds_start, ',')
			x = len(sounds)
			y = random.randint(0, x-1)
			es.cexec_all("play","1.0",sounds[y])
		else:
			es.cexec_all("play","1.0",settings.sounds_start)
	if settings.beacon:
		beacon(player1.userid)
		gamethread.delayed(settings.beacon_rate/2, beacon, player2.userid)
	start_fight(settings.timer)

def start_fight(timer):
	if timer and (status == 4):
		es.centermsg("Knife fight: %s seconds remaining!"%timer)
		timer = timer - 1
		gamethread.delayed(1, start_fight, timer)
	elif not timer and status:
		fight_timeout()
	elif timer and (status == 5):
		return
	else:
		es.msg('#multi', 'KnifeP3N: An error occured, tell Schubaal "status was 0 and timer was not"')

def fight_timeout():
	es.msg('#multi', knifep3n_msg+str_whyfight)
	es.sexec(player1.userid,"kill")
	es.sexec(player2.userid,"kill")
	kp_reset()

def fight_rewards(userid):
	global winner
	winner = playerlib.getPlayer(userid)

def player_death(event_var):
	global status, winner
	if (status == 0 and es.getlivingplayercount(2) == 1 and es.getlivingplayercount(3) == 1):
		if not settings.no_vote:
			es.msg('#multi', knifep3n_msg+str_voting)
			botList = playerlib.getUseridList('#alive,#bot')
			for botid in botList:
				bot_vote(botid)
			userList = playerlib.getUseridList('#alive,#human')
			for userid in userList:
				menu.send(userid)
			status = 1
		else:
			status = 3
			es.msg('#multi', knifep3n_msg+str_forced)
			force_fight()
	elif (status == 1 or status == 2):
		kp_reset()
	elif (status == 4):
		status = 5
		if settings.sounds_winner != "":
			if string.find(settings.sounds_winner, ','):
				sounds = string.split(settings.sounds_winner, ',')
				x = len(sounds)
				y = random.randint(0, x-1)
				es.cexec_all("play","1.0",sounds[y])
			else:
				es.cexec_all("play","1.0",settings.sounds_winner)
		if (player1.userid == int(event_var['attacker'])):
			es.msg('#multi', knifep3n_msg+player1.Player.name+str_won)
			fight_rewards(player1.userid)
		else:
			es.msg('#multi', knifep3n_msg+player2.Player.name+str_won)
			fight_rewards(player2.userid)
	elif (status == 3):
		es.msg('#multi', knifep3n_msg+playerlib.getPlayer(event_var['attacker']).name+str_cheated_knife)
		es.sexec(event_var['attacker'],"kill")